@@ -1,6 +1,7 @@
 from __future__ import annotations
 
-from functools import reduce
+import itertools
+from functools import partial, reduce
 from typing import TYPE_CHECKING
 
 import numpy as np
@@ -13,6 +14,7 @@ from flox.core import (
     _convert_expected_groups_to_index,
     _get_optimal_chunks_for_groups,
     _normalize_indexes,
+    _validate_reindex,
     factorize_,
     find_group_cohorts,
     groupby_reduce,
@@ -53,6 +55,8 @@ else:
 ALL_FUNCS = (
     "sum",
     "nansum",
+    "argmax",
+    pytest.param("nanargmax", marks=(pytest.mark.skip,)),
     "prod",
     "nanprod",
     "mean",
@@ -65,8 +69,6 @@ ALL_FUNCS = (
     "nanmax",
     "min",
     "nanmin",
-    "argmax",
-    pytest.param("nanargmax", marks=(pytest.mark.skip,)),
     "argmin",
     pytest.param("nanargmin", marks=(pytest.mark.skip,)),
     "any",
@@ -218,12 +220,26 @@ def test_groupby_reduce_all(nby, size, chunks, func, add_nan_by, engine):
             assert actual.dtype.kind == "i"
         assert_equal(actual, expected, tolerance)
 
-        if not has_dask:
+        if not has_dask or chunks is None:
             continue
-        for method in ["map-reduce", "cohorts", "split-reduce"]:
-            if "arg" in func and method != "map-reduce":
+
+        params = list(itertools.product(["map-reduce"], [True, False, None]))
+        params.extend(itertools.product(["cohorts"], [False, None]))
+        for method, reindex in params:
+            call = partial(
+                groupby_reduce, array, *by, method=method, reindex=reindex, **flox_kwargs
+            )
+            if "arg" in func and reindex is True:
+                # simple_combine with argreductions not supported right now
+                with pytest.raises(NotImplementedError):
+                    call()
                 continue
-            actual, *groups = groupby_reduce(array, *by, method=method, **flox_kwargs)
+            actual, *groups = call()
+            if "arg" not in func:
+                # make sure we use simple combine
+                assert any("simple-combine" in key for key in actual.dask.layers.keys())
+            else:
+                assert any("grouped-combine" in key for key in actual.dask.layers.keys())
             for actual_group, expect in zip(groups, expected_groups):
                 assert_equal(actual_group, expect, tolerance)
             if "arg" in func:
@@ -501,7 +517,7 @@ def test_groupby_reduce_axis_subset_against_numpy(func, axis, engine):
     assert_equal(actual, expected, tolerance)
 
 
-@pytest.mark.parametrize("chunks", [None, (2, 2, 3)])
+@pytest.mark.parametrize("reindex,chunks", [(None, None), (False, (2, 2, 3)), (True, (2, 2, 3))])
 @pytest.mark.parametrize(
     "axis, groups, expected_shape",
     [
@@ -510,7 +526,7 @@ def test_groupby_reduce_axis_subset_against_numpy(func, axis, engine):
         (None, [0], (1,)),  # global reduction; 0 shaped group axis; 1 group
     ],
 )
-def test_groupby_reduce_nans(chunks, axis, groups, expected_shape, engine):
+def test_groupby_reduce_nans(reindex, chunks, axis, groups, expected_shape, engine):
     def _maybe_chunk(arr):
         if chunks:
             if not has_dask:
@@ -533,6 +549,7 @@ def test_groupby_reduce_nans(chunks, axis, groups, expected_shape, engine):
         axis=axis,
         fill_value=0,
         engine=engine,
+        reindex=reindex,
     )
     assert_equal(result, np.zeros(expected_shape, dtype=np.intp))
 
@@ -545,7 +562,10 @@ def test_groupby_reduce_nans(chunks, axis, groups, expected_shape, engine):
 
 
 @requires_dask
-def test_groupby_all_nan_blocks(engine):
+@pytest.mark.parametrize(
+    "expected_groups, reindex", [(None, None), ([0, 1, 2], True), ([0, 1, 2], False)]
+)
+def test_groupby_all_nan_blocks(expected_groups, reindex, engine):
     labels = np.array([0, 0, 2, 2, 2, 1, 1, 2, 2, 1, 1, 0])
     nan_labels = labels.astype(float)  # copy
     nan_labels[:5] = np.nan
@@ -560,8 +580,10 @@ def test_groupby_all_nan_blocks(engine):
         da.from_array(array, chunks=(1, 3)),
         da.from_array(by, chunks=(1, 3)),
         func="sum",
-        expected_groups=None,
+        expected_groups=expected_groups,
         engine=engine,
+        reindex=reindex,
+        method="map-reduce",
     )
     assert_equal(actual, expected)
 
@@ -613,7 +635,7 @@ def test_npg_nanarg_bug(func):
     assert_equal(actual, expected)
 
 
-@pytest.mark.parametrize("method", ["split-reduce", "cohorts", "map-reduce"])
+@pytest.mark.parametrize("method", ["cohorts", "map-reduce"])
 @pytest.mark.parametrize("chunk_labels", [False, True])
 @pytest.mark.parametrize("chunks", ((), (1,), (2,)))
 def test_groupby_bins(chunk_labels, chunks, engine, method) -> None:
@@ -1125,3 +1147,39 @@ def test_subset_block_2d(flatblocks, expectidx):
     subset = subset_to_blocks(array, flatblocks)
     assert len(subset.dask.layers) == 2
     assert_equal(subset, array.compute()[expectidx])
+
+
+@pytest.mark.parametrize(
+    "expected, reindex, func, expected_groups, by_is_dask",
+    [
+        # argmax only False
+        [False, None, "argmax", None, False],
+        # True when by is numpy but expected is None
+        [True, None, "sum", None, False],
+        # False when by is dask but expected is None
+        [False, None, "sum", None, True],
+        # if expected_groups then always True
+        [True, None, "sum", [1, 2, 3], False],
+        [True, None, "sum", ([1], [2]), False],
+        [True, None, "sum", ([1], [2]), True],
+        [True, None, "sum", ([1], None), False],
+        [True, None, "sum", ([1], None), True],
+    ],
+)
+def test_validate_reindex_map_reduce(expected, reindex, func, expected_groups, by_is_dask):
+    actual = _validate_reindex(reindex, func, "map-reduce", expected_groups, by_is_dask)
+    assert actual == expected
+
+
+def test_validate_reindex():
+    for method in ["map-reduce", "cohorts"]:
+        with pytest.raises(NotImplementedError):
+            _validate_reindex(True, "argmax", method, expected_groups=None, by_is_dask=False)
+
+    for method in ["blockwise", "cohorts"]:
+        with pytest.raises(ValueError):
+            _validate_reindex(True, "sum", method, expected_groups=None, by_is_dask=False)
+
+        for func in ["sum", "argmax"]:
+            actual = _validate_reindex(None, func, method, expected_groups=None, by_is_dask=False)
+            assert actual is False
